@@ -153,7 +153,7 @@ setMethod("export", c("GenomicRanges", "BEDFile"),
                 strand <- rep(NA, length(object))
               }
               if (!is.null(strand) && is.null(score))
-                score <- 0
+                score <- rep(0, length(object))
               name <- object$name
               if (is.null(name))
                 name <- names(object)
@@ -565,7 +565,10 @@ setMethod("import", "BEDPEFile",
               gr <- callNextMethod()
               df <- mcols(gr)
               mcols(gr) <- NULL
-              df$strand2[is.na(df$strand2)] <- "*"
+              if (is.null(df$strand2))
+                  df$strand2 <- "*"
+              else
+                  df$strand2[is.na(df$strand2)] <- "*"
               seqlevels(gr) <- union(seqlevels(gr), unique(df$chrom2))
               gr2 <- with(df, GRanges(chrom2, IRanges(start2+1L, end2),
                                       strand2, seqinfo=seqinfo(gr)))
@@ -691,5 +694,6 @@ setMethod("blocks", "GenomicRanges",
             seqinfo(gr) <- seqinfo(x)
             ans <- relist(gr, mcols(x)$blocks)
             mcols(ans) <- subset(mcols(x), select=-blocks)
+            names(ans) <- if (!is.null(names(x))) names(x) else x$name
             ans
           })
